# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `ThumbnailStore.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_ModelF_ice
import omero_ServicesF_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module Glacier2
_M_Glacier2 = Ice.openModule('Glacier2')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Included module omero.grid
_M_omero.grid = Ice.openModule('omero.grid')

# Start of module omero
__name__ = 'omero'

# Start of module omero.api
__name__ = 'omero.api'

if 'ThumbnailStore' not in _M_omero.api.__dict__:
    _M_omero.api.ThumbnailStore = Ice.createTempClass()
    class ThumbnailStore(_M_omero.api.StatefulServiceInterface):
        """
        Provides methods for dealing with thumbnails. Provision is provided
        to retrieve thumbnails using the on-disk cache (provided by
        ROMIO) or on the fly.
        NOTE: The calling order for the service is as follows:
        {@code setPixelsId}
        any of the thumbnail accessor methods or
        {@code resetDefaults}
        """
        def __init__(self):
            if Ice.getType(self) == _M_omero.api.ThumbnailStore:
                raise RuntimeError('omero.api.ThumbnailStore is an abstract class')

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::api::ServiceInterface', '::omero::api::StatefulServiceInterface', '::omero::api::ThumbnailStore')

        def ice_id(self, current=None):
            return '::omero::api::ThumbnailStore'

        def ice_staticId():
            return '::omero::api::ThumbnailStore'
        ice_staticId = staticmethod(ice_staticId)

        def setPixelsId_async(self, _cb, pixelsId, current=None):
            """
            This method manages the state of the service; it must be
            invoked before using any other methods. As the
            omero.api.ThumbnailStore relies on the
            omero.api.RenderingEngine, a valid rendering
            definition must be available for it to work.
            Arguments:
            _cb -- The asynchronous callback object.
            pixelsId -- an omero.model.Pixels id.
            current -- The Current object for the invocation.
            Throws:
            ApiUsageException -- if no pixels object exists with the ID pixelsId.
            """
            pass

        def isInProgress_async(self, _cb, current=None):
            """
            This returns the last available in progress state
            for a thumbnail. Its return value is only expected
            to be valid after the call to any of the individual
            thumbnail retrieval methods.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def setRenderingDefId_async(self, _cb, renderingDefId, current=None):
            """
            This method manages the state of the service; it should be
            invoked directly after {@code setPixelsId}. If it is not
            invoked with a valid rendering definition ID before using
            the thumbnail accessor methods execution continues as if
            renderingDefId were set to null.
            Arguments:
            _cb -- The asynchronous callback object.
            renderingDefId -- an omero.model.RenderingDef id. null specifies the user's currently active rendering settings to be used.
            current -- The Current object for the invocation.
            Throws:
            ValidationException -- if no rendering definition exists with the ID renderingDefId.
            """
            pass

        def getRenderingDefId_async(self, _cb, current=None):
            """
            Return the id of the omero.model.RenderingDef
            loaded in this instance.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def getThumbnail_async(self, _cb, sizeX, sizeY, current=None):
            """
            Retrieves a thumbnail for a pixels set using a given set of
            rendering settings (RenderingDef). If the thumbnail exists
            in the on-disk cache it will be returned directly,
            otherwise it will be created as in
            {@code getThumbnailDirect}, placed in the on-disk
            cache and returned. If the thumbnail is missing, a clock will
            be returned to signify that the thumbnail is yet to be generated.
            Arguments:
            _cb -- The asynchronous callback object.
            sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
            sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
            current -- The Current object for the invocation.
            Throws:
            ApiUsageException -- if: sizeX is greater than pixels.sizeX sizeX is negative sizeY is greater than pixels.sizeY sizeY is negative {@code setPixelsId} has not yet been called
            """
            pass

        def getThumbnailWithoutDefault_async(self, _cb, sizeX, sizeY, current=None):
            """
            Retrieves a thumbnail for a pixels set using a given set of
            rendering settings (RenderingDef). If the thumbnail exists
            in the on-disk cache it will be returned directly,
            otherwise it will be created as in
            {@code getThumbnailDirect}, placed in the on-disk
            cache and returned. If the thumbnail is still to be generated, an empty array will
            be returned.
            Arguments:
            _cb -- The asynchronous callback object.
            sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
            sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
            current -- The Current object for the invocation.
            Throws:
            ApiUsageException -- if: sizeX is greater than pixels.sizeX sizeX is negative sizeY is greater than pixels.sizeY sizeY is negative {@code setPixelsId} has not yet been called
            """
            pass

        def getThumbnailSet_async(self, _cb, sizeX, sizeY, pixelsIds, current=None):
            """
            Retrieves a number of thumbnails for pixels sets using
            given sets of rendering settings (RenderingDef). If the
            thumbnails exist in the on-disk cache they will be returned
            directly, otherwise they will be created as in
            {@code getThumbnailDirect}, placed in the on-disk cache
            and returned. Unlike the other thumbnail retrieval methods,
            this method may be called without first calling
            {@code setPixelsId}.
            Arguments:
            _cb -- The asynchronous callback object.
            sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
            sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
            pixelsIds -- the Pixels sets to retrieve thumbnails for.
            current -- The Current object for the invocation.
            """
            pass

        def getThumbnailByLongestSideSet_async(self, _cb, size, pixelsIds, current=None):
            """
            Retrieves a number of thumbnails for pixels sets using
            given sets of rendering settings (RenderingDef). If the
            Thumbnails exist in the on-disk cache they will be returned
            directly, otherwise they will be created as in
            {@code getThumbnailByLongestSideDirect}. The longest
            side of the image will be used to calculate the size for
            the smaller side in order to keep the aspect ratio of the
            original image. Unlike the other thumbnail retrieval
            methods, this method may be called without first
            calling {@code setPixelsId}.
            Arguments:
            _cb -- The asynchronous callback object.
            size -- the size of the longest side of the thumbnail requested. null specifies the default size of 48.
            pixelsIds -- the Pixels sets to retrieve thumbnails for.
            current -- The Current object for the invocation.
            """
            pass

        def getThumbnailByLongestSide_async(self, _cb, size, current=None):
            """
            Retrieves a thumbnail for a pixels set using a given set of
            rendering settings (RenderingDef). If the thumbnail exists
            in the on-disk cache it will be returned directly, otherwise
            it will be created as in {@code getThumbnailDirect},
            placed in the on-disk cache and returned. The longest side
            of the image will be used to calculate the size for the
            smaller side in order to keep the aspect ratio of the
            original image.
            Arguments:
            _cb -- The asynchronous callback object.
            size -- the size of the longest side of the thumbnail requested. null specifies the default size of 48.
            current -- The Current object for the invocation.
            Throws:
            ApiUsageException -- if: size is greater than pixels.sizeX and pixels.sizeY {@code setPixelsId} has not yet been called
            """
            pass

        def getThumbnailByLongestSideDirect_async(self, _cb, size, current=None):
            """
            Retrieves a thumbnail for a pixels set using a given set of
            rendering settings (RenderingDef). The Thumbnail will
            always be created directly, ignoring the on-disk cache. The
            longest side of the image will be used to calculate the
            size for the smaller side in order to keep the aspect ratio
            of the original image.
            Arguments:
            _cb -- The asynchronous callback object.
            size -- the size of the longest side of the thumbnail requested. null specifies the default size of 48.
            current -- The Current object for the invocation.
            Throws:
            ApiUsageException -- if: size is greater than pixels.sizeX and pixels.sizeY {@code setPixelsId} has not yet been called
            """
            pass

        def getThumbnailDirect_async(self, _cb, sizeX, sizeY, current=None):
            """
            Retrieves a thumbnail for a pixels set using a given set of
            rendering settings (RenderingDef). The Thumbnail will
            always be created directly, ignoring the on-disk cache.
            Arguments:
            _cb -- The asynchronous callback object.
            sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
            sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
            current -- The Current object for the invocation.
            Throws:
            ApiUsageException -- if: sizeX is greater than pixels.sizeX sizeX is negative sizeY is greater than pixels.sizeY sizeY is negative {@code setPixelsId} has not yet been called
            """
            pass

        def getThumbnailForSectionDirect_async(self, _cb, theZ, theT, sizeX, sizeY, current=None):
            """
            Retrieves a thumbnail for a pixels set using a given set of
            rendering settings (RenderingDef) for a particular section.
            The Thumbnail will always be created directly, ignoring the
            on-disk cache.
            Arguments:
            _cb -- The asynchronous callback object.
            theZ -- the optical section (offset across the Z-axis) to use.
            theT -- the timepoint (offset across the T-axis) to use.
            sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
            sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
            current -- The Current object for the invocation.
            Throws:
            ApiUsageException -- if: sizeX is greater than pixels.sizeX sizeX is negative sizeY is greater than pixels.sizeY sizeY is negative theZ is out of range theT is out of range {@code setPixelsId} has not yet been called
            """
            pass

        def getThumbnailForSectionByLongestSideDirect_async(self, _cb, theZ, theT, size, current=None):
            """
            Retrieves a thumbnail for a pixels set using a given set of
            rendering settings (RenderingDef) for a particular section.
            The Thumbnail will always be created directly, ignoring the
            on-disk cache. The longest side of the image will be used
            to calculate the size for the smaller side in order to keep
            the aspect ratio of the original image.
            Arguments:
            _cb -- The asynchronous callback object.
            theZ -- the optical section (offset across the Z-axis) to use.
            theT -- the timepoint (offset across the T-axis) to use.
            size -- the size of the longest side of the thumbnail requested. null specifies the default size of 48.
            current -- The Current object for the invocation.
            Throws:
            ApiUsageException -- if: size is greater than pixels.sizeX and pixels.sizeY {@code setPixelsId} has not yet been called
            """
            pass

        def createThumbnails_async(self, _cb, current=None):
            """
            Creates thumbnails for a pixels set using a given set of
            rendering settings (RenderingDef) in the on-disk cache for
            every sizeX/sizeY combination already cached.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def createThumbnail_async(self, _cb, sizeX, sizeY, current=None):
            """
            Creates a thumbnail for a pixels set using a given set of
            rendering settings (RenderingDef) in the on-disk cache.
            Arguments:
            _cb -- The asynchronous callback object.
            sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
            sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
            current -- The Current object for the invocation.
            Throws:
            ApiUsageException -- if: sizeX is greater than pixels.sizeX sizeX is negative sizeY is greater than pixels.sizeY sizeY is negative {@code setPixelsId} has not yet been called
            """
            pass

        def createThumbnailsByLongestSideSet_async(self, _cb, size, pixelsIds, current=None):
            """
            Creates thumbnails for a number of pixels sets using a
            given set of rendering settings (RenderingDef) in the
            on-disk cache. Unlike the other thumbnail creation methods,
            this method may be called without first calling
            {@code setPixelsId}. This method will not reset or
            modify rendering settings in any way. If rendering settings
            for a pixels set are not present, thumbnail creation for
            that pixels set will not be performed.
            Arguments:
            _cb -- The asynchronous callback object.
            size -- the size of the longest side of the thumbnail requested. null specifies the default size of 48.
            pixelsIds -- the Pixels sets to retrieve thumbnails for.
            current -- The Current object for the invocation.
            Throws:
            ApiUsageException -- if: size is greater than pixels.sizeX and pixels.sizeY size is negative
            """
            pass

        def thumbnailExists_async(self, _cb, sizeX, sizeY, current=None):
            """
            Checks if a thumbnail of a particular size exists for a
            pixels set.
            Arguments:
            _cb -- The asynchronous callback object.
            sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
            sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
            current -- The Current object for the invocation.
            Throws:
            ApiUsageException -- if: sizeX is negative sizeY is negative setPixelsId has not yet been called
            """
            pass

        def resetDefaults_async(self, _cb, current=None):
            """
            Resets the rendering definition for the active pixels set
            to its default settings.
            Arguments:
            _cb -- The asynchronous callback object.
            current -- The Current object for the invocation.
            """
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.api._t_ThumbnailStore)

        __repr__ = __str__

    _M_omero.api.ThumbnailStorePrx = Ice.createTempClass()
    class ThumbnailStorePrx(_M_omero.api.StatefulServiceInterfacePrx):

        """
        This method manages the state of the service; it must be
        invoked before using any other methods. As the
        omero.api.ThumbnailStore relies on the
        omero.api.RenderingEngine, a valid rendering
        definition must be available for it to work.
        Arguments:
        pixelsId -- an omero.model.Pixels id.
        _ctx -- The request context for the invocation.
        Returns: true if a omero.model.RenderingDef exists for the omero.model.Pixels set, otherwise false
        Throws:
        ApiUsageException -- if no pixels object exists with the ID pixelsId.
        """
        def setPixelsId(self, pixelsId, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_setPixelsId.invoke(self, ((pixelsId, ), _ctx))

        """
        This method manages the state of the service; it must be
        invoked before using any other methods. As the
        omero.api.ThumbnailStore relies on the
        omero.api.RenderingEngine, a valid rendering
        definition must be available for it to work.
        Arguments:
        pixelsId -- an omero.model.Pixels id.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setPixelsId(self, pixelsId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_setPixelsId.begin(self, ((pixelsId, ), _response, _ex, _sent, _ctx))

        """
        This method manages the state of the service; it must be
        invoked before using any other methods. As the
        omero.api.ThumbnailStore relies on the
        omero.api.RenderingEngine, a valid rendering
        definition must be available for it to work.
        Arguments:
        pixelsId -- an omero.model.Pixels id.
        Returns: true if a omero.model.RenderingDef exists for the omero.model.Pixels set, otherwise false
        Throws:
        ApiUsageException -- if no pixels object exists with the ID pixelsId.
        """
        def end_setPixelsId(self, _r):
            return _M_omero.api.ThumbnailStore._op_setPixelsId.end(self, _r)

        """
        This returns the last available in progress state
        for a thumbnail. Its return value is only expected
        to be valid after the call to any of the individual
        thumbnail retrieval methods.
        Arguments:
        _ctx -- The request context for the invocation.
        Returns: true if the image is in the process of being imported or a pyramid is being generated for it.
        """
        def isInProgress(self, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_isInProgress.invoke(self, ((), _ctx))

        """
        This returns the last available in progress state
        for a thumbnail. Its return value is only expected
        to be valid after the call to any of the individual
        thumbnail retrieval methods.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_isInProgress(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_isInProgress.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        This returns the last available in progress state
        for a thumbnail. Its return value is only expected
        to be valid after the call to any of the individual
        thumbnail retrieval methods.
        Arguments:
        Returns: true if the image is in the process of being imported or a pyramid is being generated for it.
        """
        def end_isInProgress(self, _r):
            return _M_omero.api.ThumbnailStore._op_isInProgress.end(self, _r)

        """
        This method manages the state of the service; it should be
        invoked directly after {@code setPixelsId}. If it is not
        invoked with a valid rendering definition ID before using
        the thumbnail accessor methods execution continues as if
        renderingDefId were set to null.
        Arguments:
        renderingDefId -- an omero.model.RenderingDef id. null specifies the user's currently active rendering settings to be used.
        _ctx -- The request context for the invocation.
        Throws:
        ValidationException -- if no rendering definition exists with the ID renderingDefId.
        """
        def setRenderingDefId(self, renderingDefId, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_setRenderingDefId.invoke(self, ((renderingDefId, ), _ctx))

        """
        This method manages the state of the service; it should be
        invoked directly after {@code setPixelsId}. If it is not
        invoked with a valid rendering definition ID before using
        the thumbnail accessor methods execution continues as if
        renderingDefId were set to null.
        Arguments:
        renderingDefId -- an omero.model.RenderingDef id. null specifies the user's currently active rendering settings to be used.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_setRenderingDefId(self, renderingDefId, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_setRenderingDefId.begin(self, ((renderingDefId, ), _response, _ex, _sent, _ctx))

        """
        This method manages the state of the service; it should be
        invoked directly after {@code setPixelsId}. If it is not
        invoked with a valid rendering definition ID before using
        the thumbnail accessor methods execution continues as if
        renderingDefId were set to null.
        Arguments:
        renderingDefId -- an omero.model.RenderingDef id. null specifies the user's currently active rendering settings to be used.
        Throws:
        ValidationException -- if no rendering definition exists with the ID renderingDefId.
        """
        def end_setRenderingDefId(self, _r):
            return _M_omero.api.ThumbnailStore._op_setRenderingDefId.end(self, _r)

        """
        Return the id of the omero.model.RenderingDef
        loaded in this instance.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def getRenderingDefId(self, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_getRenderingDefId.invoke(self, ((), _ctx))

        """
        Return the id of the omero.model.RenderingDef
        loaded in this instance.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getRenderingDefId(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_getRenderingDefId.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Return the id of the omero.model.RenderingDef
        loaded in this instance.
        Arguments:
        """
        def end_getRenderingDefId(self, _r):
            return _M_omero.api.ThumbnailStore._op_getRenderingDefId.end(self, _r)

        """
        Retrieves a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef). If the thumbnail exists
        in the on-disk cache it will be returned directly,
        otherwise it will be created as in
        {@code getThumbnailDirect}, placed in the on-disk
        cache and returned. If the thumbnail is missing, a clock will
        be returned to signify that the thumbnail is yet to be generated.
        Arguments:
        sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
        sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
        _ctx -- The request context for the invocation.
        Returns: a JPEG thumbnail byte buffer.
        Throws:
        ApiUsageException -- if: sizeX is greater than pixels.sizeX sizeX is negative sizeY is greater than pixels.sizeY sizeY is negative {@code setPixelsId} has not yet been called
        """
        def getThumbnail(self, sizeX, sizeY, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_getThumbnail.invoke(self, ((sizeX, sizeY), _ctx))

        """
        Retrieves a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef). If the thumbnail exists
        in the on-disk cache it will be returned directly,
        otherwise it will be created as in
        {@code getThumbnailDirect}, placed in the on-disk
        cache and returned. If the thumbnail is missing, a clock will
        be returned to signify that the thumbnail is yet to be generated.
        Arguments:
        sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
        sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getThumbnail(self, sizeX, sizeY, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_getThumbnail.begin(self, ((sizeX, sizeY), _response, _ex, _sent, _ctx))

        """
        Retrieves a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef). If the thumbnail exists
        in the on-disk cache it will be returned directly,
        otherwise it will be created as in
        {@code getThumbnailDirect}, placed in the on-disk
        cache and returned. If the thumbnail is missing, a clock will
        be returned to signify that the thumbnail is yet to be generated.
        Arguments:
        sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
        sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
        Returns: a JPEG thumbnail byte buffer.
        Throws:
        ApiUsageException -- if: sizeX is greater than pixels.sizeX sizeX is negative sizeY is greater than pixels.sizeY sizeY is negative {@code setPixelsId} has not yet been called
        """
        def end_getThumbnail(self, _r):
            return _M_omero.api.ThumbnailStore._op_getThumbnail.end(self, _r)

        """
        Retrieves a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef). If the thumbnail exists
        in the on-disk cache it will be returned directly,
        otherwise it will be created as in
        {@code getThumbnailDirect}, placed in the on-disk
        cache and returned. If the thumbnail is still to be generated, an empty array will
        be returned.
        Arguments:
        sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
        sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
        _ctx -- The request context for the invocation.
        Returns: a JPEG thumbnail byte buffer
        Throws:
        ApiUsageException -- if: sizeX is greater than pixels.sizeX sizeX is negative sizeY is greater than pixels.sizeY sizeY is negative {@code setPixelsId} has not yet been called
        """
        def getThumbnailWithoutDefault(self, sizeX, sizeY, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_getThumbnailWithoutDefault.invoke(self, ((sizeX, sizeY), _ctx))

        """
        Retrieves a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef). If the thumbnail exists
        in the on-disk cache it will be returned directly,
        otherwise it will be created as in
        {@code getThumbnailDirect}, placed in the on-disk
        cache and returned. If the thumbnail is still to be generated, an empty array will
        be returned.
        Arguments:
        sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
        sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getThumbnailWithoutDefault(self, sizeX, sizeY, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_getThumbnailWithoutDefault.begin(self, ((sizeX, sizeY), _response, _ex, _sent, _ctx))

        """
        Retrieves a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef). If the thumbnail exists
        in the on-disk cache it will be returned directly,
        otherwise it will be created as in
        {@code getThumbnailDirect}, placed in the on-disk
        cache and returned. If the thumbnail is still to be generated, an empty array will
        be returned.
        Arguments:
        sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
        sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
        Returns: a JPEG thumbnail byte buffer
        Throws:
        ApiUsageException -- if: sizeX is greater than pixels.sizeX sizeX is negative sizeY is greater than pixels.sizeY sizeY is negative {@code setPixelsId} has not yet been called
        """
        def end_getThumbnailWithoutDefault(self, _r):
            return _M_omero.api.ThumbnailStore._op_getThumbnailWithoutDefault.end(self, _r)

        """
        Retrieves a number of thumbnails for pixels sets using
        given sets of rendering settings (RenderingDef). If the
        thumbnails exist in the on-disk cache they will be returned
        directly, otherwise they will be created as in
        {@code getThumbnailDirect}, placed in the on-disk cache
        and returned. Unlike the other thumbnail retrieval methods,
        this method may be called without first calling
        {@code setPixelsId}.
        Arguments:
        sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
        sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
        pixelsIds -- the Pixels sets to retrieve thumbnails for.
        _ctx -- The request context for the invocation.
        Returns: a map whose keys are pixels ids and values are JPEG thumbnail byte buffers or null if an exception was thrown while attempting to retrieve the thumbnail for that particular Pixels set.
        """
        def getThumbnailSet(self, sizeX, sizeY, pixelsIds, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_getThumbnailSet.invoke(self, ((sizeX, sizeY, pixelsIds), _ctx))

        """
        Retrieves a number of thumbnails for pixels sets using
        given sets of rendering settings (RenderingDef). If the
        thumbnails exist in the on-disk cache they will be returned
        directly, otherwise they will be created as in
        {@code getThumbnailDirect}, placed in the on-disk cache
        and returned. Unlike the other thumbnail retrieval methods,
        this method may be called without first calling
        {@code setPixelsId}.
        Arguments:
        sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
        sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
        pixelsIds -- the Pixels sets to retrieve thumbnails for.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getThumbnailSet(self, sizeX, sizeY, pixelsIds, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_getThumbnailSet.begin(self, ((sizeX, sizeY, pixelsIds), _response, _ex, _sent, _ctx))

        """
        Retrieves a number of thumbnails for pixels sets using
        given sets of rendering settings (RenderingDef). If the
        thumbnails exist in the on-disk cache they will be returned
        directly, otherwise they will be created as in
        {@code getThumbnailDirect}, placed in the on-disk cache
        and returned. Unlike the other thumbnail retrieval methods,
        this method may be called without first calling
        {@code setPixelsId}.
        Arguments:
        sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
        sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
        pixelsIds -- the Pixels sets to retrieve thumbnails for.
        Returns: a map whose keys are pixels ids and values are JPEG thumbnail byte buffers or null if an exception was thrown while attempting to retrieve the thumbnail for that particular Pixels set.
        """
        def end_getThumbnailSet(self, _r):
            return _M_omero.api.ThumbnailStore._op_getThumbnailSet.end(self, _r)

        """
        Retrieves a number of thumbnails for pixels sets using
        given sets of rendering settings (RenderingDef). If the
        Thumbnails exist in the on-disk cache they will be returned
        directly, otherwise they will be created as in
        {@code getThumbnailByLongestSideDirect}. The longest
        side of the image will be used to calculate the size for
        the smaller side in order to keep the aspect ratio of the
        original image. Unlike the other thumbnail retrieval
        methods, this method may be called without first
        calling {@code setPixelsId}.
        Arguments:
        size -- the size of the longest side of the thumbnail requested. null specifies the default size of 48.
        pixelsIds -- the Pixels sets to retrieve thumbnails for.
        _ctx -- The request context for the invocation.
        Returns: a map whose keys are pixels ids and values are JPEG thumbnail byte buffers or null if an exception was thrown while attempting to retrieve the thumbnail for that particular Pixels set.
        """
        def getThumbnailByLongestSideSet(self, size, pixelsIds, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_getThumbnailByLongestSideSet.invoke(self, ((size, pixelsIds), _ctx))

        """
        Retrieves a number of thumbnails for pixels sets using
        given sets of rendering settings (RenderingDef). If the
        Thumbnails exist in the on-disk cache they will be returned
        directly, otherwise they will be created as in
        {@code getThumbnailByLongestSideDirect}. The longest
        side of the image will be used to calculate the size for
        the smaller side in order to keep the aspect ratio of the
        original image. Unlike the other thumbnail retrieval
        methods, this method may be called without first
        calling {@code setPixelsId}.
        Arguments:
        size -- the size of the longest side of the thumbnail requested. null specifies the default size of 48.
        pixelsIds -- the Pixels sets to retrieve thumbnails for.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getThumbnailByLongestSideSet(self, size, pixelsIds, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_getThumbnailByLongestSideSet.begin(self, ((size, pixelsIds), _response, _ex, _sent, _ctx))

        """
        Retrieves a number of thumbnails for pixels sets using
        given sets of rendering settings (RenderingDef). If the
        Thumbnails exist in the on-disk cache they will be returned
        directly, otherwise they will be created as in
        {@code getThumbnailByLongestSideDirect}. The longest
        side of the image will be used to calculate the size for
        the smaller side in order to keep the aspect ratio of the
        original image. Unlike the other thumbnail retrieval
        methods, this method may be called without first
        calling {@code setPixelsId}.
        Arguments:
        size -- the size of the longest side of the thumbnail requested. null specifies the default size of 48.
        pixelsIds -- the Pixels sets to retrieve thumbnails for.
        Returns: a map whose keys are pixels ids and values are JPEG thumbnail byte buffers or null if an exception was thrown while attempting to retrieve the thumbnail for that particular Pixels set.
        """
        def end_getThumbnailByLongestSideSet(self, _r):
            return _M_omero.api.ThumbnailStore._op_getThumbnailByLongestSideSet.end(self, _r)

        """
        Retrieves a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef). If the thumbnail exists
        in the on-disk cache it will be returned directly, otherwise
        it will be created as in {@code getThumbnailDirect},
        placed in the on-disk cache and returned. The longest side
        of the image will be used to calculate the size for the
        smaller side in order to keep the aspect ratio of the
        original image.
        Arguments:
        size -- the size of the longest side of the thumbnail requested. null specifies the default size of 48.
        _ctx -- The request context for the invocation.
        Returns: a JPEG thumbnail byte buffer.
        Throws:
        ApiUsageException -- if: size is greater than pixels.sizeX and pixels.sizeY {@code setPixelsId} has not yet been called
        """
        def getThumbnailByLongestSide(self, size, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_getThumbnailByLongestSide.invoke(self, ((size, ), _ctx))

        """
        Retrieves a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef). If the thumbnail exists
        in the on-disk cache it will be returned directly, otherwise
        it will be created as in {@code getThumbnailDirect},
        placed in the on-disk cache and returned. The longest side
        of the image will be used to calculate the size for the
        smaller side in order to keep the aspect ratio of the
        original image.
        Arguments:
        size -- the size of the longest side of the thumbnail requested. null specifies the default size of 48.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getThumbnailByLongestSide(self, size, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_getThumbnailByLongestSide.begin(self, ((size, ), _response, _ex, _sent, _ctx))

        """
        Retrieves a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef). If the thumbnail exists
        in the on-disk cache it will be returned directly, otherwise
        it will be created as in {@code getThumbnailDirect},
        placed in the on-disk cache and returned. The longest side
        of the image will be used to calculate the size for the
        smaller side in order to keep the aspect ratio of the
        original image.
        Arguments:
        size -- the size of the longest side of the thumbnail requested. null specifies the default size of 48.
        Returns: a JPEG thumbnail byte buffer.
        Throws:
        ApiUsageException -- if: size is greater than pixels.sizeX and pixels.sizeY {@code setPixelsId} has not yet been called
        """
        def end_getThumbnailByLongestSide(self, _r):
            return _M_omero.api.ThumbnailStore._op_getThumbnailByLongestSide.end(self, _r)

        """
        Retrieves a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef). The Thumbnail will
        always be created directly, ignoring the on-disk cache. The
        longest side of the image will be used to calculate the
        size for the smaller side in order to keep the aspect ratio
        of the original image.
        Arguments:
        size -- the size of the longest side of the thumbnail requested. null specifies the default size of 48.
        _ctx -- The request context for the invocation.
        Returns: a JPEG thumbnail byte buffer.
        Throws:
        ApiUsageException -- if: size is greater than pixels.sizeX and pixels.sizeY {@code setPixelsId} has not yet been called
        """
        def getThumbnailByLongestSideDirect(self, size, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_getThumbnailByLongestSideDirect.invoke(self, ((size, ), _ctx))

        """
        Retrieves a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef). The Thumbnail will
        always be created directly, ignoring the on-disk cache. The
        longest side of the image will be used to calculate the
        size for the smaller side in order to keep the aspect ratio
        of the original image.
        Arguments:
        size -- the size of the longest side of the thumbnail requested. null specifies the default size of 48.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getThumbnailByLongestSideDirect(self, size, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_getThumbnailByLongestSideDirect.begin(self, ((size, ), _response, _ex, _sent, _ctx))

        """
        Retrieves a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef). The Thumbnail will
        always be created directly, ignoring the on-disk cache. The
        longest side of the image will be used to calculate the
        size for the smaller side in order to keep the aspect ratio
        of the original image.
        Arguments:
        size -- the size of the longest side of the thumbnail requested. null specifies the default size of 48.
        Returns: a JPEG thumbnail byte buffer.
        Throws:
        ApiUsageException -- if: size is greater than pixels.sizeX and pixels.sizeY {@code setPixelsId} has not yet been called
        """
        def end_getThumbnailByLongestSideDirect(self, _r):
            return _M_omero.api.ThumbnailStore._op_getThumbnailByLongestSideDirect.end(self, _r)

        """
        Retrieves a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef). The Thumbnail will
        always be created directly, ignoring the on-disk cache.
        Arguments:
        sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
        sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
        _ctx -- The request context for the invocation.
        Returns: a JPEG thumbnail byte buffer.
        Throws:
        ApiUsageException -- if: sizeX is greater than pixels.sizeX sizeX is negative sizeY is greater than pixels.sizeY sizeY is negative {@code setPixelsId} has not yet been called
        """
        def getThumbnailDirect(self, sizeX, sizeY, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_getThumbnailDirect.invoke(self, ((sizeX, sizeY), _ctx))

        """
        Retrieves a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef). The Thumbnail will
        always be created directly, ignoring the on-disk cache.
        Arguments:
        sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
        sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getThumbnailDirect(self, sizeX, sizeY, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_getThumbnailDirect.begin(self, ((sizeX, sizeY), _response, _ex, _sent, _ctx))

        """
        Retrieves a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef). The Thumbnail will
        always be created directly, ignoring the on-disk cache.
        Arguments:
        sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
        sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
        Returns: a JPEG thumbnail byte buffer.
        Throws:
        ApiUsageException -- if: sizeX is greater than pixels.sizeX sizeX is negative sizeY is greater than pixels.sizeY sizeY is negative {@code setPixelsId} has not yet been called
        """
        def end_getThumbnailDirect(self, _r):
            return _M_omero.api.ThumbnailStore._op_getThumbnailDirect.end(self, _r)

        """
        Retrieves a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef) for a particular section.
        The Thumbnail will always be created directly, ignoring the
        on-disk cache.
        Arguments:
        theZ -- the optical section (offset across the Z-axis) to use.
        theT -- the timepoint (offset across the T-axis) to use.
        sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
        sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
        _ctx -- The request context for the invocation.
        Returns: a JPEG thumbnail byte buffer.
        Throws:
        ApiUsageException -- if: sizeX is greater than pixels.sizeX sizeX is negative sizeY is greater than pixels.sizeY sizeY is negative theZ is out of range theT is out of range {@code setPixelsId} has not yet been called
        """
        def getThumbnailForSectionDirect(self, theZ, theT, sizeX, sizeY, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_getThumbnailForSectionDirect.invoke(self, ((theZ, theT, sizeX, sizeY), _ctx))

        """
        Retrieves a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef) for a particular section.
        The Thumbnail will always be created directly, ignoring the
        on-disk cache.
        Arguments:
        theZ -- the optical section (offset across the Z-axis) to use.
        theT -- the timepoint (offset across the T-axis) to use.
        sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
        sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getThumbnailForSectionDirect(self, theZ, theT, sizeX, sizeY, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_getThumbnailForSectionDirect.begin(self, ((theZ, theT, sizeX, sizeY), _response, _ex, _sent, _ctx))

        """
        Retrieves a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef) for a particular section.
        The Thumbnail will always be created directly, ignoring the
        on-disk cache.
        Arguments:
        theZ -- the optical section (offset across the Z-axis) to use.
        theT -- the timepoint (offset across the T-axis) to use.
        sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
        sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
        Returns: a JPEG thumbnail byte buffer.
        Throws:
        ApiUsageException -- if: sizeX is greater than pixels.sizeX sizeX is negative sizeY is greater than pixels.sizeY sizeY is negative theZ is out of range theT is out of range {@code setPixelsId} has not yet been called
        """
        def end_getThumbnailForSectionDirect(self, _r):
            return _M_omero.api.ThumbnailStore._op_getThumbnailForSectionDirect.end(self, _r)

        """
        Retrieves a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef) for a particular section.
        The Thumbnail will always be created directly, ignoring the
        on-disk cache. The longest side of the image will be used
        to calculate the size for the smaller side in order to keep
        the aspect ratio of the original image.
        Arguments:
        theZ -- the optical section (offset across the Z-axis) to use.
        theT -- the timepoint (offset across the T-axis) to use.
        size -- the size of the longest side of the thumbnail requested. null specifies the default size of 48.
        _ctx -- The request context for the invocation.
        Returns: a JPEG thumbnail byte buffer.
        Throws:
        ApiUsageException -- if: size is greater than pixels.sizeX and pixels.sizeY {@code setPixelsId} has not yet been called
        """
        def getThumbnailForSectionByLongestSideDirect(self, theZ, theT, size, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_getThumbnailForSectionByLongestSideDirect.invoke(self, ((theZ, theT, size), _ctx))

        """
        Retrieves a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef) for a particular section.
        The Thumbnail will always be created directly, ignoring the
        on-disk cache. The longest side of the image will be used
        to calculate the size for the smaller side in order to keep
        the aspect ratio of the original image.
        Arguments:
        theZ -- the optical section (offset across the Z-axis) to use.
        theT -- the timepoint (offset across the T-axis) to use.
        size -- the size of the longest side of the thumbnail requested. null specifies the default size of 48.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getThumbnailForSectionByLongestSideDirect(self, theZ, theT, size, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_getThumbnailForSectionByLongestSideDirect.begin(self, ((theZ, theT, size), _response, _ex, _sent, _ctx))

        """
        Retrieves a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef) for a particular section.
        The Thumbnail will always be created directly, ignoring the
        on-disk cache. The longest side of the image will be used
        to calculate the size for the smaller side in order to keep
        the aspect ratio of the original image.
        Arguments:
        theZ -- the optical section (offset across the Z-axis) to use.
        theT -- the timepoint (offset across the T-axis) to use.
        size -- the size of the longest side of the thumbnail requested. null specifies the default size of 48.
        Returns: a JPEG thumbnail byte buffer.
        Throws:
        ApiUsageException -- if: size is greater than pixels.sizeX and pixels.sizeY {@code setPixelsId} has not yet been called
        """
        def end_getThumbnailForSectionByLongestSideDirect(self, _r):
            return _M_omero.api.ThumbnailStore._op_getThumbnailForSectionByLongestSideDirect.end(self, _r)

        """
        Creates thumbnails for a pixels set using a given set of
        rendering settings (RenderingDef) in the on-disk cache for
        every sizeX/sizeY combination already cached.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def createThumbnails(self, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_createThumbnails.invoke(self, ((), _ctx))

        """
        Creates thumbnails for a pixels set using a given set of
        rendering settings (RenderingDef) in the on-disk cache for
        every sizeX/sizeY combination already cached.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_createThumbnails(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_createThumbnails.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Creates thumbnails for a pixels set using a given set of
        rendering settings (RenderingDef) in the on-disk cache for
        every sizeX/sizeY combination already cached.
        Arguments:
        """
        def end_createThumbnails(self, _r):
            return _M_omero.api.ThumbnailStore._op_createThumbnails.end(self, _r)

        """
        Creates a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef) in the on-disk cache.
        Arguments:
        sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
        sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
        _ctx -- The request context for the invocation.
        Throws:
        ApiUsageException -- if: sizeX is greater than pixels.sizeX sizeX is negative sizeY is greater than pixels.sizeY sizeY is negative {@code setPixelsId} has not yet been called
        """
        def createThumbnail(self, sizeX, sizeY, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_createThumbnail.invoke(self, ((sizeX, sizeY), _ctx))

        """
        Creates a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef) in the on-disk cache.
        Arguments:
        sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
        sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_createThumbnail(self, sizeX, sizeY, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_createThumbnail.begin(self, ((sizeX, sizeY), _response, _ex, _sent, _ctx))

        """
        Creates a thumbnail for a pixels set using a given set of
        rendering settings (RenderingDef) in the on-disk cache.
        Arguments:
        sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
        sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
        Throws:
        ApiUsageException -- if: sizeX is greater than pixels.sizeX sizeX is negative sizeY is greater than pixels.sizeY sizeY is negative {@code setPixelsId} has not yet been called
        """
        def end_createThumbnail(self, _r):
            return _M_omero.api.ThumbnailStore._op_createThumbnail.end(self, _r)

        """
        Creates thumbnails for a number of pixels sets using a
        given set of rendering settings (RenderingDef) in the
        on-disk cache. Unlike the other thumbnail creation methods,
        this method may be called without first calling
        {@code setPixelsId}. This method will not reset or
        modify rendering settings in any way. If rendering settings
        for a pixels set are not present, thumbnail creation for
        that pixels set will not be performed.
        Arguments:
        size -- the size of the longest side of the thumbnail requested. null specifies the default size of 48.
        pixelsIds -- the Pixels sets to retrieve thumbnails for.
        _ctx -- The request context for the invocation.
        Throws:
        ApiUsageException -- if: size is greater than pixels.sizeX and pixels.sizeY size is negative
        """
        def createThumbnailsByLongestSideSet(self, size, pixelsIds, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_createThumbnailsByLongestSideSet.invoke(self, ((size, pixelsIds), _ctx))

        """
        Creates thumbnails for a number of pixels sets using a
        given set of rendering settings (RenderingDef) in the
        on-disk cache. Unlike the other thumbnail creation methods,
        this method may be called without first calling
        {@code setPixelsId}. This method will not reset or
        modify rendering settings in any way. If rendering settings
        for a pixels set are not present, thumbnail creation for
        that pixels set will not be performed.
        Arguments:
        size -- the size of the longest side of the thumbnail requested. null specifies the default size of 48.
        pixelsIds -- the Pixels sets to retrieve thumbnails for.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_createThumbnailsByLongestSideSet(self, size, pixelsIds, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_createThumbnailsByLongestSideSet.begin(self, ((size, pixelsIds), _response, _ex, _sent, _ctx))

        """
        Creates thumbnails for a number of pixels sets using a
        given set of rendering settings (RenderingDef) in the
        on-disk cache. Unlike the other thumbnail creation methods,
        this method may be called without first calling
        {@code setPixelsId}. This method will not reset or
        modify rendering settings in any way. If rendering settings
        for a pixels set are not present, thumbnail creation for
        that pixels set will not be performed.
        Arguments:
        size -- the size of the longest side of the thumbnail requested. null specifies the default size of 48.
        pixelsIds -- the Pixels sets to retrieve thumbnails for.
        Throws:
        ApiUsageException -- if: size is greater than pixels.sizeX and pixels.sizeY size is negative
        """
        def end_createThumbnailsByLongestSideSet(self, _r):
            return _M_omero.api.ThumbnailStore._op_createThumbnailsByLongestSideSet.end(self, _r)

        """
        Checks if a thumbnail of a particular size exists for a
        pixels set.
        Arguments:
        sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
        sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
        _ctx -- The request context for the invocation.
        Throws:
        ApiUsageException -- if: sizeX is negative sizeY is negative setPixelsId has not yet been called
        """
        def thumbnailExists(self, sizeX, sizeY, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_thumbnailExists.invoke(self, ((sizeX, sizeY), _ctx))

        """
        Checks if a thumbnail of a particular size exists for a
        pixels set.
        Arguments:
        sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
        sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_thumbnailExists(self, sizeX, sizeY, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_thumbnailExists.begin(self, ((sizeX, sizeY), _response, _ex, _sent, _ctx))

        """
        Checks if a thumbnail of a particular size exists for a
        pixels set.
        Arguments:
        sizeX -- the X-axis width of the thumbnail. null specifies the default size of 48.
        sizeY -- the Y-axis width of the thumbnail. null specifies the default size of 48.
        Throws:
        ApiUsageException -- if: sizeX is negative sizeY is negative setPixelsId has not yet been called
        """
        def end_thumbnailExists(self, _r):
            return _M_omero.api.ThumbnailStore._op_thumbnailExists.end(self, _r)

        """
        Resets the rendering definition for the active pixels set
        to its default settings.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def resetDefaults(self, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_resetDefaults.invoke(self, ((), _ctx))

        """
        Resets the rendering definition for the active pixels set
        to its default settings.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_resetDefaults(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.ThumbnailStore._op_resetDefaults.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Resets the rendering definition for the active pixels set
        to its default settings.
        Arguments:
        """
        def end_resetDefaults(self, _r):
            return _M_omero.api.ThumbnailStore._op_resetDefaults.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.api.ThumbnailStorePrx.ice_checkedCast(proxy, '::omero::api::ThumbnailStore', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.api.ThumbnailStorePrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::api::ThumbnailStore'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.api._t_ThumbnailStorePrx = IcePy.defineProxy('::omero::api::ThumbnailStore', ThumbnailStorePrx)

    _M_omero.api._t_ThumbnailStore = IcePy.defineClass('::omero::api::ThumbnailStore', ThumbnailStore, -1, (), True, False, None, (_M_omero.api._t_StatefulServiceInterface,), ())
    ThumbnailStore._ice_type = _M_omero.api._t_ThumbnailStore

    ThumbnailStore._op_setPixelsId = IcePy.Operation('setPixelsId', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0),), (), ((), IcePy._t_bool, False, 0), (_M_omero._t_ServerError,))
    ThumbnailStore._op_isInProgress = IcePy.Operation('isInProgress', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), IcePy._t_bool, False, 0), (_M_omero._t_ServerError,))
    ThumbnailStore._op_setRenderingDefId = IcePy.Operation('setRenderingDefId', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_long, False, 0),), (), None, (_M_omero._t_ServerError,))
    ThumbnailStore._op_getRenderingDefId = IcePy.Operation('getRenderingDefId', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), ((), IcePy._t_long, False, 0), (_M_omero._t_ServerError,))
    ThumbnailStore._op_getThumbnail = IcePy.Operation('getThumbnail', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero._t_RInt, False, 0), ((), _M_omero._t_RInt, False, 0)), (), ((), _M_Ice._t_ByteSeq, False, 0), (_M_omero._t_ServerError,))
    ThumbnailStore._op_getThumbnailWithoutDefault = IcePy.Operation('getThumbnailWithoutDefault', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero._t_RInt, False, 0), ((), _M_omero._t_RInt, False, 0)), (), ((), _M_Ice._t_ByteSeq, False, 0), (_M_omero._t_ServerError,))
    ThumbnailStore._op_getThumbnailSet = IcePy.Operation('getThumbnailSet', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero._t_RInt, False, 0), ((), _M_omero._t_RInt, False, 0), ((), _M_omero.sys._t_LongList, False, 0)), (), ((), _M_omero.sys._t_IdByteMap, False, 0), (_M_omero._t_ServerError,))
    ThumbnailStore._op_getThumbnailByLongestSideSet = IcePy.Operation('getThumbnailByLongestSideSet', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero._t_RInt, False, 0), ((), _M_omero.sys._t_LongList, False, 0)), (), ((), _M_omero.sys._t_IdByteMap, False, 0), (_M_omero._t_ServerError,))
    ThumbnailStore._op_getThumbnailByLongestSide = IcePy.Operation('getThumbnailByLongestSide', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero._t_RInt, False, 0),), (), ((), _M_Ice._t_ByteSeq, False, 0), (_M_omero._t_ServerError,))
    ThumbnailStore._op_getThumbnailByLongestSideDirect = IcePy.Operation('getThumbnailByLongestSideDirect', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero._t_RInt, False, 0),), (), ((), _M_Ice._t_ByteSeq, False, 0), (_M_omero._t_ServerError,))
    ThumbnailStore._op_getThumbnailDirect = IcePy.Operation('getThumbnailDirect', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero._t_RInt, False, 0), ((), _M_omero._t_RInt, False, 0)), (), ((), _M_Ice._t_ByteSeq, False, 0), (_M_omero._t_ServerError,))
    ThumbnailStore._op_getThumbnailForSectionDirect = IcePy.Operation('getThumbnailForSectionDirect', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), _M_omero._t_RInt, False, 0), ((), _M_omero._t_RInt, False, 0)), (), ((), _M_Ice._t_ByteSeq, False, 0), (_M_omero._t_ServerError,))
    ThumbnailStore._op_getThumbnailForSectionByLongestSideDirect = IcePy.Operation('getThumbnailForSectionByLongestSideDirect', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), _M_omero._t_RInt, False, 0)), (), ((), _M_Ice._t_ByteSeq, False, 0), (_M_omero._t_ServerError,))
    ThumbnailStore._op_createThumbnails = IcePy.Operation('createThumbnails', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (), (), None, (_M_omero._t_ServerError,))
    ThumbnailStore._op_createThumbnail = IcePy.Operation('createThumbnail', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), _M_omero._t_RInt, False, 0), ((), _M_omero._t_RInt, False, 0)), (), None, (_M_omero._t_ServerError,))
    ThumbnailStore._op_createThumbnailsByLongestSideSet = IcePy.Operation('createThumbnailsByLongestSideSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), _M_omero._t_RInt, False, 0), ((), _M_omero.sys._t_LongList, False, 0)), (), None, (_M_omero._t_ServerError,))
    ThumbnailStore._op_thumbnailExists = IcePy.Operation('thumbnailExists', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (((), _M_omero._t_RInt, False, 0), ((), _M_omero._t_RInt, False, 0)), (), ((), IcePy._t_bool, False, 0), (_M_omero._t_ServerError,))
    ThumbnailStore._op_resetDefaults = IcePy.Operation('resetDefaults', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, True, None, (), (), (), None, (_M_omero._t_ServerError,))

    _M_omero.api.ThumbnailStore = ThumbnailStore
    del ThumbnailStore

    _M_omero.api.ThumbnailStorePrx = ThumbnailStorePrx
    del ThumbnailStorePrx

# End of module omero.api

__name__ = 'omero'

# End of module omero



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `Thumbnail.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_model_IObject_ice
import omero_RTypes_ice
import omero_model_RTypes_ice
import omero_System_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'Pixels' not in _M_omero.model.__dict__:
    _M_omero.model._t_Pixels = IcePy.declareClass('::omero::model::Pixels')
    _M_omero.model._t_PixelsPrx = IcePy.declareProxy('::omero::model::Pixels')

if 'Details' not in _M_omero.model.__dict__:
    _M_omero.model._t_Details = IcePy.declareClass('::omero::model::Details')
    _M_omero.model._t_DetailsPrx = IcePy.declareProxy('::omero::model::Details')

if 'Thumbnail' not in _M_omero.model.__dict__:
    _M_omero.model.Thumbnail = Ice.createTempClass()
    class Thumbnail(_M_omero.model.IObject):
        def __init__(self, _id=None, _details=None, _loaded=False, _version=None, _pixels=None, _mimeType=None, _sizeX=None, _sizeY=None, _ref=None):
            if Ice.getType(self) == _M_omero.model.Thumbnail:
                raise RuntimeError('omero.model.Thumbnail is an abstract class')
            _M_omero.model.IObject.__init__(self, _id, _details, _loaded)
            self._version = _version
            self._pixels = _pixels
            self._mimeType = _mimeType
            self._sizeX = _sizeX
            self._sizeY = _sizeY
            self._ref = _ref

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::IObject', '::omero::model::Thumbnail')

        def ice_id(self, current=None):
            return '::omero::model::Thumbnail'

        def ice_staticId():
            return '::omero::model::Thumbnail'
        ice_staticId = staticmethod(ice_staticId)

        def getVersion(self, current=None):
            pass

        def setVersion(self, theVersion, current=None):
            pass

        def getPixels(self, current=None):
            pass

        def setPixels(self, thePixels, current=None):
            pass

        def getMimeType(self, current=None):
            pass

        def setMimeType(self, theMimeType, current=None):
            pass

        def getSizeX(self, current=None):
            pass

        def setSizeX(self, theSizeX, current=None):
            pass

        def getSizeY(self, current=None):
            pass

        def setSizeY(self, theSizeY, current=None):
            pass

        def getRef(self, current=None):
            pass

        def setRef(self, theRef, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_Thumbnail)

        __repr__ = __str__

    _M_omero.model.ThumbnailPrx = Ice.createTempClass()
    class ThumbnailPrx(_M_omero.model.IObjectPrx):

        def getVersion(self, _ctx=None):
            return _M_omero.model.Thumbnail._op_getVersion.invoke(self, ((), _ctx))

        def begin_getVersion(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Thumbnail._op_getVersion.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getVersion(self, _r):
            return _M_omero.model.Thumbnail._op_getVersion.end(self, _r)

        def setVersion(self, theVersion, _ctx=None):
            return _M_omero.model.Thumbnail._op_setVersion.invoke(self, ((theVersion, ), _ctx))

        def begin_setVersion(self, theVersion, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Thumbnail._op_setVersion.begin(self, ((theVersion, ), _response, _ex, _sent, _ctx))

        def end_setVersion(self, _r):
            return _M_omero.model.Thumbnail._op_setVersion.end(self, _r)

        def getPixels(self, _ctx=None):
            return _M_omero.model.Thumbnail._op_getPixels.invoke(self, ((), _ctx))

        def begin_getPixels(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Thumbnail._op_getPixels.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getPixels(self, _r):
            return _M_omero.model.Thumbnail._op_getPixels.end(self, _r)

        def setPixels(self, thePixels, _ctx=None):
            return _M_omero.model.Thumbnail._op_setPixels.invoke(self, ((thePixels, ), _ctx))

        def begin_setPixels(self, thePixels, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Thumbnail._op_setPixels.begin(self, ((thePixels, ), _response, _ex, _sent, _ctx))

        def end_setPixels(self, _r):
            return _M_omero.model.Thumbnail._op_setPixels.end(self, _r)

        def getMimeType(self, _ctx=None):
            return _M_omero.model.Thumbnail._op_getMimeType.invoke(self, ((), _ctx))

        def begin_getMimeType(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Thumbnail._op_getMimeType.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getMimeType(self, _r):
            return _M_omero.model.Thumbnail._op_getMimeType.end(self, _r)

        def setMimeType(self, theMimeType, _ctx=None):
            return _M_omero.model.Thumbnail._op_setMimeType.invoke(self, ((theMimeType, ), _ctx))

        def begin_setMimeType(self, theMimeType, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Thumbnail._op_setMimeType.begin(self, ((theMimeType, ), _response, _ex, _sent, _ctx))

        def end_setMimeType(self, _r):
            return _M_omero.model.Thumbnail._op_setMimeType.end(self, _r)

        def getSizeX(self, _ctx=None):
            return _M_omero.model.Thumbnail._op_getSizeX.invoke(self, ((), _ctx))

        def begin_getSizeX(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Thumbnail._op_getSizeX.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getSizeX(self, _r):
            return _M_omero.model.Thumbnail._op_getSizeX.end(self, _r)

        def setSizeX(self, theSizeX, _ctx=None):
            return _M_omero.model.Thumbnail._op_setSizeX.invoke(self, ((theSizeX, ), _ctx))

        def begin_setSizeX(self, theSizeX, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Thumbnail._op_setSizeX.begin(self, ((theSizeX, ), _response, _ex, _sent, _ctx))

        def end_setSizeX(self, _r):
            return _M_omero.model.Thumbnail._op_setSizeX.end(self, _r)

        def getSizeY(self, _ctx=None):
            return _M_omero.model.Thumbnail._op_getSizeY.invoke(self, ((), _ctx))

        def begin_getSizeY(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Thumbnail._op_getSizeY.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getSizeY(self, _r):
            return _M_omero.model.Thumbnail._op_getSizeY.end(self, _r)

        def setSizeY(self, theSizeY, _ctx=None):
            return _M_omero.model.Thumbnail._op_setSizeY.invoke(self, ((theSizeY, ), _ctx))

        def begin_setSizeY(self, theSizeY, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Thumbnail._op_setSizeY.begin(self, ((theSizeY, ), _response, _ex, _sent, _ctx))

        def end_setSizeY(self, _r):
            return _M_omero.model.Thumbnail._op_setSizeY.end(self, _r)

        def getRef(self, _ctx=None):
            return _M_omero.model.Thumbnail._op_getRef.invoke(self, ((), _ctx))

        def begin_getRef(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Thumbnail._op_getRef.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getRef(self, _r):
            return _M_omero.model.Thumbnail._op_getRef.end(self, _r)

        def setRef(self, theRef, _ctx=None):
            return _M_omero.model.Thumbnail._op_setRef.invoke(self, ((theRef, ), _ctx))

        def begin_setRef(self, theRef, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Thumbnail._op_setRef.begin(self, ((theRef, ), _response, _ex, _sent, _ctx))

        def end_setRef(self, _r):
            return _M_omero.model.Thumbnail._op_setRef.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.ThumbnailPrx.ice_checkedCast(proxy, '::omero::model::Thumbnail', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.ThumbnailPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::Thumbnail'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_ThumbnailPrx = IcePy.defineProxy('::omero::model::Thumbnail', ThumbnailPrx)

    _M_omero.model._t_Thumbnail = IcePy.declareClass('::omero::model::Thumbnail')

    _M_omero.model._t_Thumbnail = IcePy.defineClass('::omero::model::Thumbnail', Thumbnail, -1, (), True, False, _M_omero.model._t_IObject, (), (
        ('_version', (), _M_omero._t_RInt, False, 0),
        ('_pixels', (), _M_omero.model._t_Pixels, False, 0),
        ('_mimeType', (), _M_omero._t_RString, False, 0),
        ('_sizeX', (), _M_omero._t_RInt, False, 0),
        ('_sizeY', (), _M_omero._t_RInt, False, 0),
        ('_ref', (), _M_omero._t_RString, False, 0)
    ))
    Thumbnail._ice_type = _M_omero.model._t_Thumbnail

    Thumbnail._op_getVersion = IcePy.Operation('getVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    Thumbnail._op_setVersion = IcePy.Operation('setVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    Thumbnail._op_getPixels = IcePy.Operation('getPixels', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_Pixels, False, 0), ())
    Thumbnail._op_setPixels = IcePy.Operation('setPixels', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Pixels, False, 0),), (), None, ())
    Thumbnail._op_getMimeType = IcePy.Operation('getMimeType', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Thumbnail._op_setMimeType = IcePy.Operation('setMimeType', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Thumbnail._op_getSizeX = IcePy.Operation('getSizeX', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    Thumbnail._op_setSizeX = IcePy.Operation('setSizeX', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    Thumbnail._op_getSizeY = IcePy.Operation('getSizeY', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    Thumbnail._op_setSizeY = IcePy.Operation('setSizeY', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    Thumbnail._op_getRef = IcePy.Operation('getRef', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Thumbnail._op_setRef = IcePy.Operation('setRef', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())

    _M_omero.model.Thumbnail = Thumbnail
    del Thumbnail

    _M_omero.model.ThumbnailPrx = ThumbnailPrx
    del ThumbnailPrx

# End of module omero.model

__name__ = 'omero'

# End of module omero



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Copyright 2008 Glencoe Software, Inc. All rights reserved.
# Use is subject to license terms supplied in LICENSE.txt
#

"""
   Scripting types
       - Classes:
           - Type        --  Top of parameter type hierarchy
           - Long        --
           - String      --
           - Bool        --
           - List
           - Map
           - Set
       - Functions:
           - client      -- Produces an omero.client object with given
                            input/output constraints.

"""

import os
import logging

import omero
import omero.callbacks
import omero.util.concurrency
import omero.util.temp_files

from omero.rtypes import rint, rfloat, rstring, rinternal, rbool, rmap
from omero.rtypes import robject, rlist, rset, rtype, rlong, rdouble
from omero.rtypes import wrap, unwrap

from collections import defaultdict


BASE_LOG = logging.getLogger("omero.scripts")
PROC_LOG = logging.getLogger("omero.scripts.ProcessCallback")
TYPE_LOG = logging.getLogger("omero.scripts.Type")


class Type(omero.grid.Param):
    """
    omero.grid.Param subclass which provides convenience methods for
    input/output specification.
    Further subclasses are responsible for creating proper prototypes.

    kwargs
    """
    PROTOTYPE_FUNCTION = None
    PROTOTYPE_DEFAULT = None
    PROTOTYPE_MIN = None
    PROTOTYPE_MAX = None
    PROTOTYPE_VALUES = None

    def __init__(self, name, optional=True, out=False, description=None,
                 default=None, **kwargs):

        # Non-Param attributes
        omero.grid.Param.__init__(self)

        # Non-Param attributes
        self._name = name
        self._in = True
        self._out = out

        # Other values will be filled in by the kwargs
        # Mostly leaving these for backwards compatibility
        self.description = description
        self.optional = optional

        # Assign all the kwargs
        for k, v in list(kwargs.items()):
            if not hasattr(self, k):
                TYPE_LOG.warn("Unknown property: %s", k)
            setattr(self, k, v)

        _DEF = self.__get(self.PROTOTYPE_DEFAULT, False)
        _FUN = self.__get(self.PROTOTYPE_FUNCTION)
        _MAX = self.__get(self.PROTOTYPE_MAX)
        _MIN = self.__get(self.PROTOTYPE_MIN)
        _VAL = self.__get(self.PROTOTYPE_VALUES)

        # Someone specifically set the prototype, then
        # we assume that useDefault should be True
        # For whatever reason, inheritance isn't working.
        if default is not None:
            newfunc = _FUN
            newdefault = default
            if isinstance(self, List):
                if isinstance(default, (list, tuple)):
                    newdefault = wrap(default).val
                elif isinstance(default, omero.RCollection):
                    newfunc = lambda x: x
                elif isinstance(default, omero.RType):
                    default = [default]
                else:
                    newfunc = lambda x: x
                    newdefault = rlist([rtype(default)])
            self.useDefault = True
            self.prototype = newfunc(newdefault)
        else:
            if not callable(_FUN):
                raise ValueError("Bad prototype function: %s" % _FUN)

            # To prevent weirdness, if the class default is
            # callable, we'll assume its a constructor and
            # create a new one to prevent modification.
            try:
                _def = _DEF()
            except TypeError:
                _def = _DEF
            self.prototype = _FUN(_def)

        # The following use wrap to guarantee that an rtype is present
        if self.min is not None:
            if _MIN is None:
                self.min = _FUN(self.min)
            else:
                self.min = _MIN(self.min)

        if self.max is not None:
            if _MAX is None:
                self.max = _FUN(self.max)
            else:
                self.max = _MAX(self.max)

        if self.values is not None:
            if _VAL is None:
                self.values = wrap(self.values)
            else:
                self.values = _VAL(self.values)

        # Now if useDefault has been set, either manually, or
        # via setting default="..." we check that the default
        # value matches a value if present
        if self.values is not None and self.values and self.useDefault:
            if isinstance(self.prototype, omero.RCollection):
                test = unwrap(self.prototype.val[0])
            else:
                test = unwrap(self.prototype)
            values = unwrap(self.values)
            if test not in values:
                raise ValueError("%s is not in %s" % (test, values))

    def out(self):
        self._in = False
        self._out = True
        return self

    def inout(self):
        self._in = True
        self._out = True
        return self

    def type(self, *arg):
        self.prototype = wrap(arg)
        return self

    def name(self):
        return self._name

    def __get(self, val, func=True):
        if val is not None:
            if func:
                return val.__func__
            else:
                return val


class Object(Type):
    """
    Wraps an robject
    """
    PROTOTYPE_FUNCTION = robject
    PROTOTYPE_DEFAULT = None


class Long(Type):
    """
    Wraps an rlong
    """
    PROTOTYPE_FUNCTION = rlong
    PROTOTYPE_DEFAULT = 0


class Int(Type):
    """
    Wraps an rint
    """
    PROTOTYPE_FUNCTION = rint
    PROTOTYPE_DEFAULT = 0


class Double(Type):
    """
    Wraps an rdouble
    """
    PROTOTYPE_FUNCTION = rdouble
    PROTOTYPE_DEFAULT = 0.0


class Float(Type):
    """
    Wraps an rfloat
    """
    PROTOTYPE_FUNCTION = rfloat
    PROTOTYPE_DEFAULT = 0.0


class String(Type):
    """
    Wraps an rstring
    """
    PROTOTYPE_FUNCTION = rstring
    PROTOTYPE_DEFAULT = ""


class Bool(Type):
    """
    Wraps an rbool
    """
    PROTOTYPE_FUNCTION = rbool
    PROTOTYPE_DEFAULT = False


class Color(Type):
    """
    Wraps an rinternal(Color)
    """
    PROTOTYPE_FUNCTION = rinternal
    PROTOTYPE_DEFAULT = omero.Color


class Point(Type):
    """
    Wraps an rinternal(Point)
    """
    PROTOTYPE_FUNCTION = rinternal
    PROTOTYPE_DEFAULT = omero.Point


class Plane(Type):
    """
    Wraps an rinternal(Plane)
    """
    PROTOTYPE_FUNCTION = rinternal
    PROTOTYPE_DEFAULT = omero.Plane


class __Coll(Type):
    """
    Base type providing the append and extend functionality.
    Not for user use.
    """
    PROTOTYPE_DEFAULT = list

    def append(self, *arg):
        self.prototype.val.append(*arg)
        return self

    def extend(self, *arg):
        self.prototype.val.extend(*arg)
        return self

    def ofType(self, obj):
        if callable(obj):
            obj = obj()  # Ctors, etc.

        # If someone used default=, then "ofType()" is not necessary
        # and so we check for their correspondence.
        if self.useDefault and self.prototype.val:
            if not isinstance(obj, self.prototype.val[0].__class__):
                raise ValueError(
                    "ofType values doesn't match default value: %s <> %s"
                    % (unwrap(obj), unwrap(self.prototype.val[0])))
        else:
            self.prototype.val.append(wrap(obj))

        return self


class Set(__Coll):
    """
    Wraps an rset. To add values to the contents of the set,
    use "append" or "extend" since set.val is of type list.
    """
    PROTOTYPE_FUNCTION = rset


class List(__Coll):
    """
    Wraps an rlist. To add values to the contents of the list,
    use "append" or "extend" since set.val is of type list.
    """
    PROTOTYPE_FUNCTION = rlist


class Map(Type):
    """
    Wraps an rmap. To add values to the contents of the map,
    use "update" since map.val is of type dict.
    """
    PROTOTYPE_FUNCTION = rmap
    PROTOTYPE_DEFAULT = dict

    def update(self, *args, **kwargs):
        self.prototype.val.update(*args, **kwargs)
        return self


class ParseExit(Exception):
    """
    Raised when this script should just parse parameters and return.
    """

    def __init__(self, params):
        Exception.__init__(self)
        self.params = params


def client(*args, **kwargs):
    """
    Entry point for all script engine scripts.

    Typical usage consists of::

        client = omero.scripts.client("name","description", \
            omero.scripts.Long("name"),...)

    where the returned client is created via the empty constructor to
    omero.client using only --Ice.Config or ICE_CONFIG, and the function
    arguments are taken as metadata about the current script. With this
    information, all script consumers should be able to determine the required
    types for execution.

    Possible types are all subclasses of omero.scripts.Type

    To change the omero.model.Format of the stdout and stderr produced by
    this script, use the constructor arguments::

        client = omero.scripts.client(..., \
            stdoutFormat = "text/plain",
            stderrFormat = "text/plain")

    If you would like to prevent stdout and/or stderr from being
    uploaded, set the corresponding value to None. If you would like
    to overwrite the value with another file, use
    client.setOutput(). Though it is possible to attach any RType to
    "stdout" or "stderr", there is an assumption that the value will
    be an robject(OriginalFileI())

    Providing your own client is possible via the kwarg "client = ...",
    but be careful since this may break usage with the rest of the
    scripting framework. The client should not have a session, and
    must be configured for the argumentless version of createSession()
    """

    args = list(args)
    if len(args) >= 1:
        if isinstance(args[0], str):
            kwargs["name"] = args.pop(0)
    if len(args) >= 1:
        if isinstance(args[0], str):
            kwargs["description"] = args.pop(0)

    if "client" not in kwargs:
        c = omero.client()
        router = c.getProperty("Ice.Default.Router")
        router = c.getCommunicator().stringToProxy(router)
        for endpoint in router.ice_getEndpoints():
            host = endpoint.getInfo().host
            c.ic.getProperties().setProperty("omero.host", host)
            break
        else:
            BASE_LOG.warn("no host configuration found for script client")
        kwargs["client"] = c
    else:
        c = kwargs["client"]
    c.setAgent("OMERO.scripts")

    if args and isinstance(args[0], omero.grid.JobParams):
        c.params = args.pop(0)
    else:
        c.params = omero.grid.JobParams()
        c.params.inputs = {}
        c.params.outputs = {}

    for k, v in list(kwargs.items()):
        if hasattr(c.params, k):
            setattr(c.params, k, v)

    if not c.params.stdoutFormat:
        c.params.stdoutFormat = "text/plain"

    if not c.params.stderrFormat:
        c.params.stderrFormat = "text/plain"

    for p in args:
        if isinstance(p, Type):
            if p._in:
                c.params.inputs[p._name] = p
            if p._out:
                c.params.outputs[p._name] = p
        else:
            raise ValueError("Not Type: %s" % type(p))

    handleParse(c)  # May throw

    c.createSession().detachOnDestroy()
    return c


def handleParse(c):
    """
    Raises ParseExit if the client has the configuration property
    "omero.scripts.parse". If the value is anything other than "only",
    then the parameters will also be sent to the server.
    """
    parse = c.getProperty("omero.scripts.parse")
    if len(parse) > 0:  # TODO Add to omero/Constants.ice
        if parse != "only":
            c.createSession().detachOnDestroy()
            c.setOutput("omero.scripts.parse", rinternal(c.params))
        raise ParseExit(c.params)


def parse_text(scriptText):
    """
    Parses the given script text with "omero.scripts.parse" set
    and catches the exception. The parameters are returned.

    WARNING: This method calls "exec" on the given text.
    Do NOT use this on data you don't trust.
    """
    try:
        cfg = omero.util.temp_files.create_path()
        cfg.write_lines(["omero.scripts.parse=only", "omero.host=localhost"])
        old = os.environ.get("ICE_CONFIG")
        try:
            os.environ["ICE_CONFIG"] = cfg.abspath()
            exec(scriptText, {"__name__": "__main__"})
            raise Exception("Did not throw ParseExit: %s" % scriptText)
        finally:
            if old:
                os.environ["ICE_CONFIG"] = old
    except ParseExit as exit:
        return exit.params


def parse_file(filename):
    """
    Parses the given script file with "omero.scripts.parse" set
    and catches the exception. The parameters are returned.

    WARNING: This method calls "exec" on the given file's contents.
    Do NOT use this on data you don't trust.
    """
    from omero_ext.path import path
    scriptText = path(filename).text()
    return parse_text(scriptText)


class MissingInputs(Exception):
    def __init__(self):
        Exception.__init__(self)
        self.keys = []


def parse_inputs(inputs_strings, params):
    """
    Parses a command-line like string representation of input parameters
    into an RDict (a map from string to RType). The input should be an
    iterable of arguments of the form "key=value".

    For example, "a=1" gets mapped to {"a":1}
    """
    inputs = {}
    for input_string in inputs_strings:
        rv = parse_input(input_string, params)
        inputs.update(rv)

    missing = MissingInputs()
    for key in sorted(
            params.inputs, key=lambda name: params.inputs.get(name).grouping):
        param = params.inputs.get(key)
        a = inputs.get(key, None)
        if not a:
            if param.useDefault:
                inputs[key] = param.prototype
            elif not param.optional:
                missing.keys.append(key)

    if missing.keys:
        missing.inputs = inputs
        raise missing

    return inputs


def parse_input(input_string, params):
    """
    Parse a single input_string. The params
    """
    parts = input_string.split("=")
    if len(parts) == 1:
        parts.append("")
    key = parts[0]
    val = parts[1]

    param = params.inputs.get(key)
    if param is None:
        return {}
    elif isinstance(param.prototype, omero.RBool):
        if val.lower() in ("false", "False", "0", ""):
            val = rbool(False)
        else:
            val = rbool(True)
    elif isinstance(
        param.prototype,
        (omero.RLong, omero.RString, omero.RInt,
         omero.RTime, omero.RDouble, omero.RFloat)):
        val = param.prototype.__class__(val)
    elif isinstance(param.prototype, (omero.RList, omero.RSet)):
        rmethod = omero.rtypes.rlist
        rwrap = omero.rtypes.wrap
        if isinstance(param.prototype, omero.RSet):
            rmethod = omero.rtypes.rset
        if len(param.prototype.val) > 0:
            rwrap = param.prototype.val[0].__class__
        items = val.split(",")
        val = rmethod([rwrap(x) for x in items])
    elif isinstance(param.prototype, omero.RObject):
        try:
            parts2 = val.split(":")
            kls = parts2[0]
            _id = int(parts2[1])
            if not kls.endswith("I"):
                kls = "%sI" % kls
            kls = getattr(omero.model, kls)
        except:
            raise ValueError(
                "Format for objects: Class:id or ClassI:id. Not:%s" % val)
        val = omero.rtypes.robject(kls(_id, False))
    else:
        raise ValueError("No converter for: %s (type=%s)"
                         % (key, param.prototype.__class__))

    return {key: val}


def group_params(params):
    """
    Walks through the inputs of the given JobParams
    and returns a map-of-maps with Param names as
    the leaf nodes.

    For example, for the following:

        Params("1", grouping = "A") # "A." is equivalent
        Params("2", grouping = "A.B")
        Params("3", grouping = "A.C")

    this function returns:

        {"A" : {"": "1", "B" : "2", "C" : "3"} }

    while:

        Params("1", grouping = "A")

    returns:

        {"A" : "1"}

    """
    nested_dict = lambda: defaultdict(nested_dict)
    groupings = nested_dict()
    ordered_params = sorted(list(params.inputs.items()))
    for param_name, v in ordered_params:

        val = v.grouping
        if val.endswith("."):
            val = val[:-1]

        parts = val.split(".")

        previous = None
        current = groupings
        for idx, key in enumerate(parts):
            if (idx+1) < len(parts):
                # We need to descend further
                previous = current
                current = current[key]
            else:
                # No further keys remaining, assign
                if isinstance(current, dict):
                    if key in current:
                        current[key][""] = param_name
                    else:
                        current[key] = param_name
                elif isinstance(current, str):
                    # Here we assume the value is a node key
                    replacement = dict()
                    replacement[""] = current
                    replacement[key] = param_name
                    assert previous[parts[idx-1]] == current
                    previous[parts[idx-1]] = replacement
                else:
                    raise Exception(current, type(current))

    return groupings


def error_msg(category, key, format_string, *args):
    c = "%s" % (category.upper())
    s = """%s for "%s": %s\n""" % (c, key, format_string)
    return s % args


def compare_proto(key, proto, input, cache=None):

    if cache is None:
        cache = {}

    if id(proto) in cache and id(input) in cache:
        return ""  # Prevent StackOverflow
    else:
        cache[id(proto)] = True
        cache[id(input)] = True

    itype = None
    ptype = None
    both_collection = False
    if input is not None:
        itype = input.__class__
        both_collection = isinstance(input, omero.RCollection)
    if proto is not None:
        ptype = proto.__class__
        both_collection &= isinstance(proto, omero.RCollection)

    # see https://github.com/openmicroscopy/openmicroscopy/issues/5788
    # accept RSets as RLists and vice versa
    if not (both_collection or isinstance(input, ptype)):
        return error_msg("Wrong type", key, "%s != %s", itype, ptype)

    # Now recurse if a collection type
    errors = ""
    if isinstance(proto, omero.RMap) and len(proto.val) > 0:
        for x in list(input.val.values()):
            errors += compare_proto(key, list(proto.val.values())[0], x, cache)
    elif isinstance(proto, omero.RCollection) and len(proto.val) > 0:
        for x in input.val:
            errors += compare_proto(key, proto.val[0], x, cache)
    return errors


def expand(input):
    if input is None:
        items = []
    elif isinstance(input, (list, tuple)):
        items = list(input)
    elif isinstance(input, dict):
        items = list(input.values())
    else:
        items = [input]
    return items


def check_boundaries(key, min, max, input):
    errors = ""

    # Unwrap
    min = unwrap(min)
    max = unwrap(max)
    input = unwrap(input)
    items = expand(input)

    # Check
    for x in items:
        if min is not None and min > x:
            errors += error_msg("Out of bounds", key, "%s is below min %s", x,
                                min)
        if max is not None and max < x:
            errors += error_msg("Out of bounds", key, "%s is above max %s", x,
                                max)
    return errors


def check_values(key, values, input):
    errors = ""

    # Unwrap
    values = unwrap(values)
    input = unwrap(input)
    items = expand(input)
    values = expand(values)

    if not values:
        return errors

    for x in items:
        if x not in values:
            errors += error_msg("Value list", key, "%s not in %s", x, values)

    return errors


def validate_inputs(params, inputs, svc=None, session=None):
    """
    Method used by processor.py to check the input values
    provided by the user launching the script. If a non-empty
    errors string is returned, then the inputs fail validation.

    A service instance can be provided in order to add default
    values to the session. If no service instance is provided,
    values with a default which are missing will be counted as
    errors.
    """
    errors = ""
    for key, param in list(params.inputs.items()):
        if key not in inputs:
            if param.optional:
                if param.useDefault and svc is not None:
                    set_input(svc, session, key, param.prototype)
            else:  # Not optional
                if param.useDefault:
                    errors += set_input(svc, session, key, param.prototype)
                else:
                    errors += error_msg("Missing input", key, "")
        else:
            input = inputs[key]
            errors += compare_proto(key, param.prototype, input)
            errors += check_boundaries(key, param.min, param.max, input)
            errors += check_values(key, param.values, input)
    return errors


def set_input(svc, session, key, value):
    try:
        svc.setInput(session, key, value)
        return ""
    except Exception as e:
        return error_msg("Failed to set intput", key, "%s=%s. Error: %s", key,
                         value, e)

#
# Importing into omero.scripts namespace
#
ProcessCallbackI = omero.callbacks.ProcessCallbackI


def wait(client, process, ms=500):
    """
    Wrapper around the use of ProcessCallbackI
    """
    cb = ProcessCallbackI(client, process)
    while cb.block(500) is None:
        process.poll()
